@@ -29,12 +29,12 @@
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
+#include <xfconf/xfconf.h>
 #include <libxfce4windowing/libxfce4windowing.h>
 
 #include <stdarg.h>
 
 #define XFDESKTOP_CHANNEL  "xfce4-desktop"
-#define DEFAULT_BACKDROP   DATADIR "/backgrounds/xfce/xfce-shapes.svg"
 
 #define MIN_ICON_SIZE               16
 #define MAX_ICON_SIZE              192
@@ -49,7 +49,8 @@
 #define MAX_GRAVITY                  7
 #define DEFAULT_GRAVITY              0
 
-#define DEFAULT_SINGLE_CLICK       FALSE
+#define DEFAULT_SINGLE_CLICK        FALSE
+#define DEFAULT_SINGLE_CLICK_ULINE  FALSE
 #define DEFAULT_ICONS_ON_PRIMARY    FALSE
 #define DEFAULT_ICON_FONT_SIZE_SET  FALSE
 #define DEFAULT_ICON_CENTER_TEXT    TRUE
@@ -79,12 +80,14 @@
 #define DESKTOP_ICONS_SHOW_TOOLTIP_PROP      "/desktop-icons/show-tooltips"
 #define DESKTOP_ICONS_TOOLTIP_SIZE_PROP      "/desktop-icons/tooltip-size"
 #define DESKTOP_ICONS_SINGLE_CLICK_PROP      "/desktop-icons/single-click"
+#define DESKTOP_ICONS_SINGLE_CLICK_ULINE_PROP "/desktop-icons/single-click-underline-hover"
 #define DESKTOP_ICONS_GRAVITY_PROP           "/desktop-icons/gravity"
 
 #define DESKTOP_ICONS_SHOW_THUMBNAILS        "/desktop-icons/show-thumbnails"
 #define DESKTOP_ICONS_SHOW_HIDDEN_FILES      "/desktop-icons/show-hidden-files"
 #define DESKTOP_ICONS_SHOW_NETWORK_REMOVABLE "/desktop-icons/file-icons/show-network-removable"
 #define DESKTOP_ICONS_SHOW_DEVICE_REMOVABLE  "/desktop-icons/file-icons/show-device-removable"
+#define DESKTOP_ICONS_SHOW_DEVICE_FIXED      "/desktop-icons/file-icons/show-device-fixed"
 #define DESKTOP_ICONS_SHOW_UNKNWON_REMOVABLE "/desktop-icons/file-icons/show-unknown-removable"
 #define DESKTOP_ICONS_SHOW_HOME              "/desktop-icons/file-icons/show-home"
 #define DESKTOP_ICONS_SHOW_TRASH             "/desktop-icons/file-icons/show-trash"
@@ -115,8 +118,65 @@
 #define XFDESKTOP_FILESYSTEM_INFO_NAMESPACE \
   "filesystem::*"
 
+#define XFCE_TYPE_BACKDROP_COLOR_STYLE (xfce_backdrop_color_style_get_type())
+#define XFCE_TYPE_BACKDROP_IMAGE_STYLE (xfce_backdrop_image_style_get_type())
+#define XFCE_TYPE_BACKDROP_CYCLE_PERIOD (xfce_backdrop_cycle_period_get_type())
+#define XFCE_TYPE_DESKTOP_ICON_STYLE (xfce_desktop_icon_style_get_type())
+
+
+#ifdef ENABLE_DESKTOP_ICONS
+#ifdef ENABLE_FILE_ICONS
+#define ICON_STYLE_DEFAULT XFCE_DESKTOP_ICON_STYLE_FILES
+#else  /* !ENABLE_FILE_ICONS */
+#define ICON_STYLE_DEFAULT XFCE_DESKTOP_ICON_STYLE_WINDOWS
+#endif /* ENABLE_FILE_ICONS */
+#else  /* !ENABLE_DESKTOP_ICONS */
+#define ICON_STYLE_DEFAULT XFCE_DESKTOP_ICON_STYLE_NONE
+#endif  /* ENABLE_DESKTOP_ICONS */
+
 G_BEGIN_DECLS
 
+typedef enum {
+    XFCE_BACKDROP_COLOR_INVALID = -1,
+    XFCE_BACKDROP_COLOR_SOLID = 0,
+    XFCE_BACKDROP_COLOR_HORIZ_GRADIENT,
+    XFCE_BACKDROP_COLOR_VERT_GRADIENT,
+    XFCE_BACKDROP_COLOR_TRANSPARENT,
+} XfceBackdropColorStyle;
+
+typedef enum {
+    XFCE_BACKDROP_IMAGE_INVALID = -1,
+    XFCE_BACKDROP_IMAGE_NONE = 0,
+    XFCE_BACKDROP_IMAGE_CENTERED,
+    XFCE_BACKDROP_IMAGE_TILED,
+    XFCE_BACKDROP_IMAGE_STRETCHED,
+    XFCE_BACKDROP_IMAGE_SCALED,
+    XFCE_BACKDROP_IMAGE_ZOOMED,
+    XFCE_BACKDROP_IMAGE_SPANNING_SCREENS,
+} XfceBackdropImageStyle;
+
+typedef enum {
+    XFCE_BACKDROP_PERIOD_INVALID = -1,
+    XFCE_BACKDROP_PERIOD_SECONDS = 0,
+    XFCE_BACKDROP_PERIOD_MINUTES,
+    XFCE_BACKDROP_PERIOD_HOURS,
+    XFCE_BACKDROP_PERIOD_STARTUP,
+    XFCE_BACKDROP_PERIOD_HOURLY,
+    XFCE_BACKDROP_PERIOD_DAILY,
+    XFCE_BACKDROP_PERIOD_CHRONOLOGICAL,
+} XfceBackdropCyclePeriod;
+
+typedef enum {
+    XFCE_DESKTOP_ICON_STYLE_NONE = 0,
+    XFCE_DESKTOP_ICON_STYLE_WINDOWS,
+    XFCE_DESKTOP_ICON_STYLE_FILES,
+} XfceDesktopIconStyle;
+
+GType xfce_backdrop_color_style_get_type(void) G_GNUC_CONST;
+GType xfce_backdrop_image_style_get_type(void) G_GNUC_CONST;
+GType xfce_backdrop_cycle_period_get_type(void) G_GNUC_CONST;
+GType xfce_desktop_icon_style_get_type(void) G_GNUC_CONST;
+
 gchar* xfdesktop_get_monitor_name_from_gtk_widget(GtkWidget *widget,
                                                   gint monitor_num);
 
@@ -136,10 +196,6 @@ GtkWidget* xfdesktop_menu_create_menu_item_with_markup(const gchar *name,
 GtkWidget* xfdesktop_menu_create_menu_item_with_mnemonic(const gchar *name,
                                                          GtkWidget   *image);
 
-void xfdesktop_get_screen_dimensions(GdkScreen *gcreen,
-                                     gint      *width,
-                                     gint      *height);
-
 gint xfdesktop_get_monitor_num(GdkDisplay *display,
                                GdkMonitor *monitor);
 
@@ -177,6 +233,8 @@ void xfdesktop_debug(const char *func, const char *file, int line, const char *f
 
 void xfdesktop_debug_set(gboolean debug);
 
+void xfdesktop_migrate_backdrop_settings(GdkDisplay *display,
+                                         XfconfChannel *channel);
 G_END_DECLS
 
 #endif
