@@ -55,14 +55,21 @@
 #include <libxfce4util/libxfce4util.h>
 #include <xfconf/xfconf.h>
 #include <libxfce4ui/libxfce4ui.h>
+#include <libxfce4kbd-private/xfce-shortcuts-editor.h>
 #include <libxfce4windowing/libxfce4windowing.h>
+#ifdef ENABLE_X11
+#include <libxfce4windowing/xfw-x11.h>
+#endif  /* ENABLE_X11 */
 #include <exo/exo.h>
 
-#include "xfdesktop-common.h"
-#include "xfdesktop-thumbnailer.h"
+#include "common/xfdesktop-common.h"
+#include "common/xfdesktop-keyboard-shortcuts.h"
+#include "common/xfdesktop-thumbnailer.h"
 #include "xfdesktop-settings-ui.h"
-/* for XfceBackdropImageStyle && XfceBackdropColorStyle */
-#include "xfce-backdrop.h"
+
+#ifdef HAVE_XFCE_REVISION_H
+#include "xfce-revision.h"
+#endif
 
 #define MAX_ASPECT_RATIO 1.5f
 #define PREVIEW_HEIGHT   96
@@ -163,6 +170,7 @@ enum
     COL_ICON_NAME,
     COL_ICON_ENABLED,
     COL_ICON_PROPERTY,
+    COL_ICON_SENSITIVE,
     N_ICON_COLS,
 };
 
@@ -174,7 +182,7 @@ static gchar *xfdesktop_settings_get_backdrop_image(AppearancePanel *panel);
 
 
 
-static const gchar *
+static gchar *
 system_data_lookup (void)
 {
     const gchar * const * dirs;
@@ -399,6 +407,17 @@ cb_special_icon_toggled(GtkCellRendererToggle *render, gchar *path, gpointer use
     gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
                        COL_ICON_ENABLED, show_icon, -1);
 
+    if (g_strcmp0(icon_property, DESKTOP_ICONS_SHOW_REMOVABLE) == 0) {
+        GtkTreeIter child_iter;
+        if (gtk_tree_model_iter_children(model, &child_iter, &iter)) {
+            do {
+                gtk_tree_store_set(GTK_TREE_STORE(model), &child_iter,
+                                   COL_ICON_SENSITIVE, show_icon,
+                                   -1);
+            } while (gtk_tree_model_iter_next(model, &child_iter));
+        }
+    }
+
     gtk_tree_path_free(tree_path);
     g_free(icon_property);
 }
@@ -411,7 +430,7 @@ setup_special_icon_list(GtkBuilder *gxml,
     GtkTreeStore *ts;
     GtkTreeViewColumn *col;
     GtkCellRenderer *render;
-    GtkTreeIter iter, parent_iter, child_iter;
+    GtkTreeIter *iter, parent_iter, child_iter;
     const struct {
         const gchar *name;
         const gchar *icon_names[2];
@@ -424,12 +443,14 @@ setup_special_icon_list(GtkBuilder *gxml,
           DESKTOP_ICONS_SHOW_FILESYSTEM, TRUE },
         { N_("Trash"), { "user-trash", "gnome-fs-trash-empty" },
           DESKTOP_ICONS_SHOW_TRASH, TRUE },
-        { N_("Removable Devices"), { "drive-removable-media", "gnome-dev-removable" },
+        { N_("Devices"), { "drive-harddisk", "drive-removable-media" },
           DESKTOP_ICONS_SHOW_REMOVABLE, TRUE },
         { N_("Network Shares"), { "gtk-network", "gnome-dev-network" },
           DESKTOP_ICONS_SHOW_NETWORK_REMOVABLE, TRUE },
-        { N_("Disks and Drives"), { "drive-harddisk-usb", "gnome-dev-removable-usb" },
+        { N_("Removable Disks and Drives"), { "drive-harddisk-usb", "gnome-dev-removable-usb" },
           DESKTOP_ICONS_SHOW_DEVICE_REMOVABLE, TRUE },
+        { N_("Fixed Disks and Drives"), { "drive-harddisk-system", "drive-harddisk", },
+          DESKTOP_ICONS_SHOW_DEVICE_FIXED, TRUE },
         { N_("Other Devices"), { "multimedia-player", "phone" },
           DESKTOP_ICONS_SHOW_UNKNWON_REMOVABLE, TRUE },
         { NULL, { NULL, NULL }, NULL, FALSE },
@@ -439,27 +460,32 @@ setup_special_icon_list(GtkBuilder *gxml,
 
     gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, NULL);
 
-    ts = gtk_tree_store_new(N_ICON_COLS, G_TYPE_ICON, G_TYPE_STRING,
-                            G_TYPE_BOOLEAN, G_TYPE_STRING);
+    gboolean removable_devices_enabled = xfconf_channel_get_bool(channel, DESKTOP_ICONS_SHOW_REMOVABLE, TRUE);
+
+    ts = gtk_tree_store_new(N_ICON_COLS,
+                            G_TYPE_ICON,
+                            G_TYPE_STRING,
+                            G_TYPE_BOOLEAN,
+                            G_TYPE_STRING,
+                            G_TYPE_BOOLEAN);
     for(i = 0; icons[i].name; ++i) {
         GIcon *icon = g_themed_icon_new_from_names((char **)icons[i].icon_names, G_N_ELEMENTS(icons[i].icon_names));
 
         if(i < REMOVABLE_DEVICES) {
             gtk_tree_store_append(ts, &parent_iter, NULL);
-            iter = parent_iter;
+            iter = &parent_iter;
         } else {
             gtk_tree_store_append(ts, &child_iter, &parent_iter);
-            iter = child_iter;
+            iter = &child_iter;
         }
 
-        gtk_tree_store_set(ts, &iter,
+        gboolean sensitive = iter == &parent_iter || removable_devices_enabled;
+        gtk_tree_store_set(ts, iter,
                            COL_ICON_NAME, _(icons[i].name),
                            COL_ICON_PIX, icon,
                            COL_ICON_PROPERTY, icons[i].xfconf_property,
-                           COL_ICON_ENABLED,
-                           xfconf_channel_get_bool(channel,
-                                                   icons[i].xfconf_property,
-                                                   icons[i].state),
+                           COL_ICON_ENABLED, xfconf_channel_get_bool(channel, icons[i].xfconf_property, icons[i].state),
+                           COL_ICON_SENSITIVE, sensitive,
                            -1);
         if (icon != NULL)
             g_object_unref(icon);
@@ -474,6 +500,7 @@ setup_special_icon_list(GtkBuilder *gxml,
     render = gtk_cell_renderer_toggle_new();
     gtk_tree_view_column_pack_start(col, render, FALSE);
     gtk_tree_view_column_add_attribute(col, render, "active", COL_ICON_ENABLED);
+    gtk_tree_view_column_add_attribute(col, render, "sensitive", COL_ICON_SENSITIVE);
 
     g_signal_connect(G_OBJECT(render), "toggled",
                      G_CALLBACK(cb_special_icon_toggled), treeview);
@@ -481,13 +508,17 @@ setup_special_icon_list(GtkBuilder *gxml,
     render = gtk_cell_renderer_pixbuf_new();
     gtk_tree_view_column_pack_start(col, render, FALSE);
     gtk_tree_view_column_add_attribute(col, render, "gicon", COL_ICON_PIX);
+    gtk_tree_view_column_add_attribute(col, render, "sensitive", COL_ICON_SENSITIVE);
 
     render = gtk_cell_renderer_text_new();
     gtk_tree_view_column_pack_start(col, render, TRUE);
     gtk_tree_view_column_add_attribute(col, render, "text", COL_ICON_NAME);
+    gtk_tree_view_column_add_attribute(col, render, "sensitive", COL_ICON_SENSITIVE);
 
     gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(ts));
     g_object_unref(G_OBJECT(ts));
+
+    gtk_tree_view_expand_all(GTK_TREE_VIEW(treeview));
 }
 
 
@@ -1673,7 +1704,16 @@ cb_workspace_changed(XfwWorkspaceGroup *group,
 }
 
 static void
-cb_workspace_count_changed(XfwWorkspaceGroup *group,
+cb_workspace_group_created(XfwWorkspaceManager *manager,
+                           XfwWorkspaceGroup *group,
+                           AppearancePanel *panel)
+{
+    g_signal_connect(group, "active-workspace-changed",
+                     G_CALLBACK(cb_workspace_changed), panel);
+}
+
+static void
+cb_workspace_count_changed(XfwWorkspaceManager *manager,
                            XfwWorkspace *workspace,
                            gpointer user_data)
 {
@@ -1696,7 +1736,7 @@ is_our_window(AppearancePanel *panel,
     if (xfw_windowing_get() == XFW_WINDOWING_X11) {
         GdkWindow *toplevel_window = gtk_widget_get_window(toplevel);
         Window our_xid = gdk_x11_window_get_xid(toplevel_window);
-        Window window_xid = xfw_window_get_id(window);
+        Window window_xid = xfw_window_x11_get_xid(window);
         Window cur_win = our_xid;
         GdkDisplay *gdpy = gdk_window_get_display(toplevel_window);
         Display *dpy = gdk_x11_display_get_xdisplay(gdpy);
@@ -1808,6 +1848,7 @@ cb_window_opened(XfwScreen *screen,
     xfdesktop_settings_update_iconview_frame_name(panel, workspace);
 }
 
+#ifdef ENABLE_X11
 static void
 cb_active_window_changed(XfwScreen *screen,
                          XfwWindow *previously_active_window,
@@ -1823,18 +1864,21 @@ cb_active_window_changed(XfwScreen *screen,
         }
     }
 }
+#endif
 
 static void
 cb_monitor_changed(GdkScreen *gscreen,
                    gpointer user_data)
 {
-    AppearancePanel *panel = user_data;
+    if (gdk_display_get_n_monitors(gdk_screen_get_display(gscreen)) > 0) {
+        AppearancePanel *panel = user_data;
 
-    /* Update background because the monitor we're on may have changed */
-    cb_update_background_tab(panel->xfw_window, user_data);
+        /* Update background because the monitor we're on may have changed */
+        cb_update_background_tab(panel->xfw_window, user_data);
 
-    /* Update the frame name because we may change from/to a single monitor */
-    xfdesktop_settings_update_iconview_frame_name(panel, xfw_window_get_workspace(panel->xfw_window));
+        /* Update the frame name because we may change from/to a single monitor */
+        xfdesktop_settings_update_iconview_frame_name(panel, xfw_window_get_workspace(panel->xfw_window));
+    }
 }
 
 static void
@@ -1910,8 +1954,11 @@ cb_icon_style_changed(GtkComboBox *combo,
                              gtk_combo_box_get_active(combo) != 0);
 
 #ifdef ENABLE_FILE_ICONS
-    gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(main_gxml, "box_file_icons_settings")),
+    gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(main_gxml, "tab_file_launcher_icons_content")),
                              gtk_combo_box_get_active(combo) == 2);
+    gtk_widget_set_visible(GTK_WIDGET(gtk_builder_get_object(main_gxml, "infobar_file_launcher_icons_disabled")),
+                           gtk_combo_box_get_active(combo) != 2);
+
 #endif
 }
 
@@ -1945,13 +1992,65 @@ cb_xfdesktop_icon_orientation_changed(GtkComboBox *combo,
     }
 }
 
+static void
+add_keyboard_tab_widgets(GtkContainer *parent) {
+    GArray *editor_sections = g_array_sized_new(FALSE, TRUE, sizeof(XfceShortcutsEditorSection), 3);
+
+    gsize n_desktop_action_entries;
+    XfceGtkActionEntry *desktop_action_entries = xfdesktop_get_desktop_actions(NULL, &n_desktop_action_entries);
+    XfceShortcutsEditorSection section = {
+        .section_name = N_("Desktop"),
+        .entries = desktop_action_entries,
+        .size = n_desktop_action_entries,
+    };
+    g_array_append_val(editor_sections, section);
+
+#ifdef ENABLE_DESKTOP_ICONS
+    gsize n_icon_view_entries;
+    XfceGtkActionEntry *icon_view_action_entries = xfdesktop_get_icon_view_actions(NULL, &n_icon_view_entries);
+    section = (XfceShortcutsEditorSection){
+        .section_name = N_("Icons"),
+        .entries = icon_view_action_entries,
+        .size = n_icon_view_entries,
+    };
+    g_array_append_val(editor_sections, section);
+
+    gsize n_window_icon_manager_entries;
+    XfceGtkActionEntry *window_icon_manager_action_entries = xfdesktop_get_window_icon_manager_actions(NULL, &n_window_icon_manager_entries);
+    section = (XfceShortcutsEditorSection){
+        .section_name = N_("Window Icons"),
+        .entries = window_icon_manager_action_entries,
+        .size = n_window_icon_manager_entries,
+    };
+    g_array_append_val(editor_sections, section);
+
+#ifdef ENABLE_FILE_ICONS
+    gsize n_file_icon_manager_entries;
+    XfceGtkActionEntry *file_icon_manager_entries = xfdesktop_get_file_icon_manager_actions(NULL, &n_file_icon_manager_entries);
+    section = (XfceShortcutsEditorSection){
+        .section_name = N_("File Icons"),
+        .entries = file_icon_manager_entries,
+        .size = n_file_icon_manager_entries,
+    };
+    g_array_append_val(editor_sections, section);
+#endif /* ENABLE_FILE_ICONS */
+#endif /* ENABLE_DESKTOP_ICONS */
+
+    gsize n_sections = editor_sections->len;
+    XfceShortcutsEditorSection *sections = (XfceShortcutsEditorSection *)(gpointer)g_array_free(editor_sections, FALSE);
+    GtkWidget *shortcuts_editor = xfce_shortcuts_editor_new_array(sections, n_sections);
+    gtk_container_add(parent, shortcuts_editor);
+    gtk_widget_show_all(shortcuts_editor);
+}
+
 static void
 xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
                                      AppearancePanel *panel)
 {
     GtkWidget *appearance_container, *chk_custom_font_size,
               *spin_font_size, *w, *box, *spin_icon_size,
-              *chk_show_thumbnails, *chk_single_click, *appearance_settings,
+              *chk_show_thumbnails, *chk_single_click, *chk_single_click_underline,
+              *appearance_settings,
               *chk_show_tooltips, *spin_tooltip_size, *bnt_exit, *content_area,
               *chk_show_hidden_files;
     GtkBuilder *appearance_gxml;
@@ -1961,8 +2060,8 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     XfwScreen *xfw_screen;
     XfwWorkspaceManager *workspace_manager;
     XfconfChannel *channel = panel->channel;
-    const gchar *path;
     GFile *file;
+    gchar *path;
     gchar *uri_path;
 
     TRACE("entering");
@@ -1976,6 +2075,10 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
                      G_CALLBACK(cb_xfdesktop_bnt_exit_clicked),
                      panel);
 
+    // Keyboard tab
+    GtkWidget *keyboard_tab = GTK_WIDGET(gtk_builder_get_object(main_gxml, "tab_keyboard_shortcuts"));
+    add_keyboard_tab_widgets(GTK_CONTAINER(keyboard_tab));
+
     /* Icons tab */
     /* icon size */
     spin_icon_size = GTK_WIDGET(gtk_builder_get_object(main_gxml, "spin_icon_size"));
@@ -1998,6 +2101,11 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     /* single click */
     chk_single_click = GTK_WIDGET(gtk_builder_get_object(main_gxml,
                                                          "chk_single_click"));
+    chk_single_click_underline = GTK_WIDGET(gtk_builder_get_object(main_gxml,
+                                                                   "chk_single_click_underline"));
+    g_object_bind_property(chk_single_click, "active",
+                           chk_single_click_underline, "sensitive",
+                           G_BINDING_SYNC_CREATE);
 
     /* show hidden files */
     chk_show_hidden_files = GTK_WIDGET(gtk_builder_get_object(main_gxml,
@@ -2031,17 +2139,18 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     screen = gtk_widget_get_screen(appearance_container);
     xfw_screen = xfw_screen_get_default();
     workspace_manager = xfw_screen_get_workspace_manager(xfw_screen);
-    // FIXME: watch for new and destroyed workspace groups
+    g_signal_connect(workspace_manager, "workspace-group-created",
+                     G_CALLBACK(cb_workspace_group_created), panel);
+    g_signal_connect(workspace_manager, "workspace-created",
+                     G_CALLBACK(cb_workspace_count_changed), panel);
+    g_signal_connect(workspace_manager, "workspace-destroyed",
+                     G_CALLBACK(cb_workspace_count_changed), panel);
 
     /* watch for workspace changes */
     for (GList *l = xfw_workspace_manager_list_workspace_groups(workspace_manager);
          l != NULL;
          l = l->next)
     {
-        g_signal_connect(l->data, "workspace-created",
-                         G_CALLBACK(cb_workspace_count_changed), panel);
-        g_signal_connect(l->data, "workspace-destroyed",
-                         G_CALLBACK(cb_workspace_count_changed), panel);
         g_signal_connect(l->data, "active-workspace-changed",
                          G_CALLBACK(cb_workspace_changed), panel);
     }
@@ -2054,8 +2163,11 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     g_signal_connect(G_OBJECT(screen), "monitors-changed",
                      G_CALLBACK(cb_monitor_changed), panel);
 
-    appearance_gxml = gtk_builder_new_from_resource("/org/xfce/xfdesktop/settings/xfdesktop-settings-appearance-frame-ui.glade");
-    if (G_UNLIKELY(appearance_gxml == NULL)) {
+    appearance_gxml = gtk_builder_new();
+    if (gtk_builder_add_from_resource(appearance_gxml,
+                                      "/org/xfce/xfdesktop/settings/xfdesktop-settings-appearance-frame-ui.glade",
+                                      &error) == 0)
+    {
         g_printerr("Failed to parse appearance settings UI description: %s\n",
                    error->message);
         g_clear_error(&error);
@@ -2120,6 +2232,7 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
         gtk_file_chooser_add_shortcut_folder_uri (GTK_FILE_CHOOSER(panel->btn_folder),
                                                   uri_path, NULL);
 
+        g_free (path);
         g_free (uri_path);
         g_object_unref (file);
     }
@@ -2245,7 +2358,7 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     gtk_combo_box_set_active(GTK_COMBO_BOX(w), 2);
 #else
     gtk_combo_box_set_active(GTK_COMBO_BOX(w), 1);
-    gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(main_gxml, "box_file_icons_settings")));
+    gtk_notebook_remove_page(GTK_NOTEBOOK(gtk_builder_get_object(main_gxml, "notebook_settings")), 3);
 #endif
     xfconf_g_property_bind(channel, DESKTOP_ICONS_STYLE_PROP, G_TYPE_INT,
                            G_OBJECT(w), "active");
@@ -2283,6 +2396,9 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     xfconf_g_property_bind(channel, DESKTOP_ICONS_SINGLE_CLICK_PROP,
                            G_TYPE_BOOLEAN, G_OBJECT(chk_single_click),
                            "active");
+    xfconf_g_property_bind(channel, DESKTOP_ICONS_SINGLE_CLICK_ULINE_PROP,
+                           G_TYPE_BOOLEAN, G_OBJECT(chk_single_click_underline),
+                           "active");
 
     setup_special_icon_list(main_gxml, channel);
     cb_update_background_tab(panel->xfw_window, panel);
@@ -2296,7 +2412,7 @@ xfdesktop_settings_response(GtkWidget *dialog, gint response_id, gpointer user_d
                                            "xfdesktop",
                                            "start",
                                            NULL,
-                                           XFDESKTOP_VERSION_SHORT);
+                                           VERSION_SHORT);
     } else {
         XfconfChannel *channel = (XfconfChannel*) user_data;
         GdkWindowState state;
@@ -2316,6 +2432,11 @@ xfdesktop_settings_response(GtkWidget *dialog, gint response_id, gpointer user_d
     }
 }
 
+static void
+accel_map_changed(void) {
+    xfdesktop_keyboard_shortcuts_save();
+}
+
 static Window opt_socket_id = 0;
 static gboolean opt_version = FALSE;
 static gboolean opt_enable_debug = FALSE;
@@ -2323,7 +2444,7 @@ static GOptionEntry option_entries[] = {
     { "socket-id", 's', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_INT, &opt_socket_id, N_("Settings manager socket"), N_("SOCKET ID") },
     { "version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_version, N_("Version information"), NULL },
     { "enable-debug", 'e', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_enable_debug, N_("Enable debug messages"), NULL },
-    { NULL, },
+    { NULL, '\0', 0, 0, NULL, NULL, NULL },
 };
 
 int
@@ -2356,7 +2477,7 @@ main(int argc, char **argv)
     }
 
     if(G_UNLIKELY(opt_version)) {
-        g_print("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, VERSION, xfce_version_string());
+        g_print("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, VERSION_FULL, xfce_version_string());
         g_print("%s\n", "Copyright (c) 2004-2022");
         g_print("\t%s\n\n", _("The Xfce development team. All rights reserved."));
         g_print(_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
@@ -2378,8 +2499,12 @@ main(int argc, char **argv)
 
     xfdesktop_settings_ui_register_resource();
 
-    gxml = gtk_builder_new_from_resource("/org/xfce/xfdesktop/settings/xfdesktop-settings-ui.glade");
-    if (G_UNLIKELY(gxml == NULL)) {
+    xfdesktop_keyboard_shortcuts_init();
+    g_signal_connect(gtk_accel_map_get(), "changed",
+                     G_CALLBACK(accel_map_changed), NULL);
+
+    gxml = gtk_builder_new();
+    if (gtk_builder_add_from_resource(gxml, "/org/xfce/xfdesktop/settings/xfdesktop-settings-ui.glade", &error) == 0) {
         g_printerr("Failed to parse UI description: %s\n", error->message);
         g_clear_error(&error);
         return 1;
@@ -2462,6 +2587,8 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
     g_object_unref(G_OBJECT(gxml));
 
+    xfdesktop_keyboard_shortcuts_shutdown();
+
     g_object_unref(G_OBJECT(channel));
     xfconf_shutdown();
 
