@@ -598,7 +598,7 @@ invalidate_backdrops_for_property_prefix(XfdesktopBackdropManager *manager, cons
 
 static void
 channel_property_changed(XfdesktopBackdropManager *manager, const gchar *property_name, const GValue *value) {
-    DBG("entering(%s)", property_name);
+    DBG("entering");
 
     const gchar *last_slash = g_strrstr(property_name, "/");
     if (last_slash != NULL) {
@@ -806,8 +806,7 @@ render_finished(cairo_surface_t *surface, gint width, gint height, GError *error
             if (backdrop == NULL) {
                 backdrop = g_new0(Backdrop, 1);
                 backdrop->manager = rdata->manager;
-                backdrop->cycler = xfdesktop_backdrop_cycler_new(rdata->manager->channel, rdata->property_prefix,
-                                                                 rdata->image_file);
+                backdrop->cycler = xfdesktop_backdrop_cycler_new(rdata->manager->channel, rdata->property_prefix);
                 g_hash_table_insert(rdata->manager->backdrops, rdata->property_prefix, backdrop);
                 rdata->property_prefix = NULL;
             } else {
