@@ -70,7 +70,6 @@ struct _XfdesktopFileIconModel
     GHashTable *icons;
 
     gboolean show_thumbnails;
-    gboolean sort_folders_before_files;
 };
 
 struct _XfdesktopFileIconModelClass
@@ -84,7 +83,6 @@ typedef enum {
     PROP_CHANNEL,
     PROP_FOLDER,
     PROP_SHOW_THUMBNAILS,
-    PROP_SORT_FOLDERS_BEFORE_FILES,
 } ModelPropertyId;
 
 enum {
@@ -156,14 +154,6 @@ G_DEFINE_QUARK("xfdesktop-file-icon-model-error-quark", xfdesktop_file_icon_mode
 
 static guint signals[N_SIGS] = { 0, };
 
-static const struct {
-    const gchar *setting;
-    GType setting_type;
-    const gchar *property;
-} setting_bindings[] = {
-    { DESKTOP_ICONS_SHOW_THUMBNAILS, G_TYPE_BOOLEAN, "show-thumbnails" },
-    { DESKTOP_ICONS_SORT_FOLDERS_BEFORE_FILES_PROP, G_TYPE_BOOLEAN, "sort-folders-before-files" },
-};
 
 static void
 xfdesktop_file_icon_model_class_init(XfdesktopFileIconModelClass *klass)
@@ -202,13 +192,6 @@ xfdesktop_file_icon_model_class_init(XfdesktopFileIconModelClass *klass)
                                                          "show-thumbnails",
                                                          TRUE,
                                                          G_PARAM_READWRITE));
-    g_object_class_install_property(gobject_class,
-                                    PROP_SORT_FOLDERS_BEFORE_FILES,
-                                    g_param_spec_boolean("sort-folders-before-files",
-                                                         "sort-folders-before-files",
-                                                         "sort-folders-before-files",
-                                                         TRUE,
-                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     XfdesktopIconViewModelClass *ivmodel_class = XFDESKTOP_ICON_VIEW_MODEL_CLASS(klass);
     ivmodel_class->model_item_ref = g_object_ref;
@@ -260,7 +243,6 @@ xfdesktop_file_icon_model_tree_model_init(GtkTreeModelIface *iface)
 static void
 xfdesktop_file_icon_model_init(XfdesktopFileIconModel *fmodel) {
     fmodel->show_thumbnails = TRUE;
-    fmodel->sort_folders_before_files = TRUE;
     fmodel->icons = g_hash_table_new_full(g_str_hash,
                                           g_str_equal,
                                           g_free,
@@ -295,13 +277,7 @@ xfdesktop_file_icon_model_constructed(GObject *object) {
     g_signal_connect(fmodel->volume_monitor, "mount-pre-unmount",
                      G_CALLBACK(mount_pre_unmount), fmodel);
 
-    for (gsize i = 0; i < G_N_ELEMENTS(setting_bindings); ++i) {
-        xfconf_g_property_bind(fmodel->channel,
-                               setting_bindings[i].setting,
-                               setting_bindings[i].setting_type,
-                               fmodel,
-                               setting_bindings[i].property);
-    }
+    xfconf_g_property_bind(fmodel->channel, DESKTOP_ICONS_SHOW_THUMBNAILS, G_TYPE_BOOLEAN, fmodel, "show-thumbnails");
 }
 
 static void
@@ -325,10 +301,6 @@ xfdesktop_file_icon_model_set_property(GObject *object, guint property_id, const
             xfdesktop_file_icon_model_set_show_thumbnails(fmodel, g_value_get_boolean(value));
             break;
 
-        case PROP_SORT_FOLDERS_BEFORE_FILES:
-            fmodel->sort_folders_before_files = g_value_get_boolean(value);
-            break;
-
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -355,11 +327,6 @@ xfdesktop_file_icon_model_get_property(GObject *object, guint property_id, GValu
         case PROP_SHOW_THUMBNAILS:
             g_value_set_boolean(value, fmodel->show_thumbnails);
             break;
-
-        case PROP_SORT_FOLDERS_BEFORE_FILES:
-            g_value_set_boolean(value, fmodel->sort_folders_before_files);
-            break;
-
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -472,18 +439,9 @@ xfdesktop_file_icon_model_get_value(GtkTreeModel *model,
             } else if (XFDESKTOP_IS_VOLUME_ICON(icon)) {
                 priority = 1;
             } else if (XFDESKTOP_IS_REGULAR_FILE_ICON(icon)) {
-                if (XFDESKTOP_FILE_ICON_MODEL(model)->sort_folders_before_files) {
-                    GFileInfo *file_info = xfdesktop_file_icon_peek_file_info(icon);
-                    if (g_file_info_get_file_type(file_info) == G_FILE_TYPE_DIRECTORY) {
-                        priority = 2;
-                    } else {
-                        priority = 3;
-                    }
-                } else {
-                    priority = 2;
-                }
+                priority = 2;
             } else {
-                priority = 4;
+                priority = 3;
             }
 
             g_value_init(value, G_TYPE_INT);
@@ -553,9 +511,6 @@ check_create_desktop_folder(XfdesktopFileIconModel *fmodel) {
         g_signal_emit(fmodel, signals[SIG_ERROR], 0, error);
         result = FALSE;
         g_error_free(error);
-    }
-
-    if (info != NULL) {
         g_object_unref(info);
     }
 
@@ -577,7 +532,6 @@ add_icon(XfdesktopFileIconModel *fmodel, XfdesktopFileIcon *icon) {
     g_signal_connect_swapped(icon, "pixbuf-changed",
                              G_CALLBACK(xfdesktop_icon_view_model_changed), fmodel);
 
-    XF_DEBUG("adding icon %s to icon view", xfdesktop_icon_peek_label(XFDESKTOP_ICON(icon)));
     g_hash_table_replace(fmodel->icons, g_strdup(xfdesktop_file_icon_peek_sort_key(icon)), icon);
     xfdesktop_icon_view_model_append(XFDESKTOP_ICON_VIEW_MODEL(fmodel), icon, icon, NULL);
 }
@@ -681,7 +635,6 @@ mount_added(GVolumeMonitor *monitor, GMount *mount, XfdesktopFileIconModel *fmod
                 DBG("got existing icon for volume for mount");
                 xfdesktop_volume_icon_mounted(XFDESKTOP_VOLUME_ICON(icon), mount);
                 xfdesktop_icon_view_model_changed(XFDESKTOP_ICON_VIEW_MODEL(fmodel), icon);
-                g_hash_table_insert(fmodel->volume_icons, g_object_ref(mount), icon);
             }
             g_object_unref(volume);
         } else {
