@@ -31,7 +31,7 @@
 
 #include "xfdesktop-icon-position-configs.h"
 
-#define SAVE_DELAY_S 1
+#define SAVE_DELAY_S 3
 
 #ifdef DEBUG
 static char spaces[512];
@@ -266,7 +266,6 @@ save_icons(XfdesktopIconPositionConfigs *configs, GError **error) {
                     new_filename,
                     strerror(errno));
         yaml_emitter_delete(&emitter);
-        g_free(new_filename);
         return FALSE;
     }
 
@@ -486,7 +485,6 @@ G_STMT_END
         goto out_err;
     }
 
-    g_free(new_filename);
     return TRUE;
 
 out_err:
@@ -516,9 +514,10 @@ save_icons_idled(gpointer data) {
 
 static void
 schedule_save(XfdesktopIconPositionConfigs *configs) {
-    if (configs->scheduled_save_id == 0) {
-        configs->scheduled_save_id = g_timeout_add_seconds(SAVE_DELAY_S, save_icons_idled, configs);
+    if (configs->scheduled_save_id != 0) {
+        g_source_remove(configs->scheduled_save_id);
     }
+    configs->scheduled_save_id = g_timeout_add_seconds(SAVE_DELAY_S, save_icons_idled, configs);
 }
 
 static gboolean
@@ -1224,8 +1223,8 @@ xfdesktop_icon_position_configs_set_icon_position(XfdesktopIconPositionConfigs *
 
     for (GList *l = configs->configs; l != NULL; l = l->next) {
         XfdesktopIconPositionConfig *a_config = l->data;
-        if (a_config != config && a_config->level >= config->level) {
-            DBG("removing icon from higher or equal prio config");
+        if (a_config->level > config->level) {
+            DBG("removing icon from higher-prio config");
             // XXX: Do we want to do this for all configs, or only assigned
             // configs?  I could make an argument either way.
             g_hash_table_remove(a_config->icon_positions, identifier);
