@@ -22,6 +22,7 @@
 #define __XFDESKTOP_ICON_VIEW_H__
 
 #include <gtk/gtk.h>
+#include <libxfce4windowing/libxfce4windowing.h>
 #include <xfconf/xfconf.h>
 
 G_BEGIN_DECLS
@@ -43,7 +44,7 @@ typedef enum
 
 struct _XfdesktopIconView
 {
-    GtkWidget parent;
+    GtkEventBox parent;
 
     /*< private >*/
     XfdesktopIconViewPrivate *priv;
@@ -51,7 +52,7 @@ struct _XfdesktopIconView
 
 struct _XfdesktopIconViewClass
 {
-    GtkWidgetClass parent;
+    GtkEventBoxClass parent;
 
     /*< signals >*/
     void (*icon_selection_changed)(XfdesktopIconView *icon_view);
@@ -72,14 +73,9 @@ struct _XfdesktopIconViewClass
                               gint new_cols);
     void (*end_grid_resize)(XfdesktopIconView *icon_view);
 
-    void (*select_all)(XfdesktopIconView *icon_view);
-    void (*unselect_all)(XfdesktopIconView *icon_view);
-
     void (*select_cursor_item)(XfdesktopIconView *icon_view);
     void (*toggle_cursor_item)(XfdesktopIconView *icon_view);
 
-    gboolean (*activate_selected_items)(XfdesktopIconView *icon_view);
-
     gboolean (*move_cursor)(XfdesktopIconView *icon_view,
                             GtkMovementStep step,
                             gint count);
@@ -87,10 +83,20 @@ struct _XfdesktopIconViewClass
     void (*resize_event)(XfdesktopIconView *icon_view);
 };
 
+typedef struct {
+    XfdesktopIconView *source_icon_view;
+    GList *dragged_icons;  // GtkTreeIter
+} XfdesktopDraggedIconList;
+
 GType xfdesktop_icon_view_get_type(void) G_GNUC_CONST;
 
-GtkWidget *xfdesktop_icon_view_new(XfconfChannel *channel) G_GNUC_WARN_UNUSED_RESULT;
+guint xfdesktop_icon_view_get_icon_drag_info(void);
+GdkAtom xfdesktop_icon_view_get_icon_drag_target(void);
+
+GtkWidget *xfdesktop_icon_view_new(XfconfChannel *channel,
+                                   XfwScreen *screen) G_GNUC_WARN_UNUSED_RESULT;
 GtkWidget *xfdesktop_icon_view_new_with_model(XfconfChannel *channel,
+                                              XfwScreen *screen,
                                               GtkTreeModel *model) G_GNUC_WARN_UNUSED_RESULT;
 
 void xfdesktop_icon_view_set_model(XfdesktopIconView *icon_view,
@@ -99,6 +105,8 @@ GtkTreeModel *xfdesktop_icon_view_get_model(XfdesktopIconView *icon_view);
 
 void xfdesktop_icon_view_set_pixbuf_column(XfdesktopIconView *icon_view,
                                            gint column);
+void xfdesktop_icon_view_set_icon_opacity_column(XfdesktopIconView *icon_view,
+                                                 gint column);
 void xfdesktop_icon_view_set_text_column(XfdesktopIconView *icon_view,
                                          gint column);
 void xfdesktop_icon_view_set_search_column(XfdesktopIconView *icon_view,
@@ -130,10 +138,33 @@ void xfdesktop_icon_view_enable_drag_dest(XfdesktopIconView *icon_view,
 void xfdesktop_icon_view_unset_drag_source(XfdesktopIconView *icon_view);
 void xfdesktop_icon_view_unset_drag_dest(XfdesktopIconView *icon_view);
 
+void xfdesktop_icon_view_draw_highlight(XfdesktopIconView *icon_view,
+                                        gint row,
+                                        gint col);
+void xfdesktop_icon_view_unset_highlight(XfdesktopIconView *icon_view);
+
+GtkTargetList *xfdesktop_icon_view_get_drag_dest_targets(XfdesktopIconView *icon_view);
+
 gboolean xfdesktop_icon_view_widget_coords_to_item(XfdesktopIconView *icon_view,
                                                    gint wx,
                                                    gint wy,
                                                    GtkTreeIter *iter);
+gboolean xfdesktop_icon_view_widget_coords_to_slot_coords(XfdesktopIconView *icon_view,
+                                                          gint wx,
+                                                          gint wy,
+                                                          gint *row,
+                                                          gint *col);
+
+gboolean xfdesktop_icon_view_slot_coords_to_widget_coords(XfdesktopIconView *icon_view,
+                                                          gint row,
+                                                          gint col,
+                                                          gint *wx,
+                                                          gint *wy);
+
+gboolean xfdesktop_icon_view_get_cursor(XfdesktopIconView *icon_view,
+                                        GtkTreeIter *iter,
+                                        gint *row,
+                                        gint *col);
 
 GList *xfdesktop_icon_view_get_selected_items(XfdesktopIconView *icon_view) G_GNUC_WARN_UNUSED_RESULT;
 
@@ -168,6 +199,9 @@ gboolean xfdesktop_icon_view_get_single_click(XfdesktopIconView *icon_view);
 void xfdesktop_icon_view_set_single_click(XfdesktopIconView *icon_view,
                                           gboolean single_click);
 
+void xfdesktop_icon_view_set_single_click_underline_hover(XfdesktopIconView *icon_view,
+                                                          gboolean single_click_underline_hover);
+
 void xfdesktop_icon_view_set_gravity(XfdesktopIconView *icon_view,
                                      XfdesktopIconViewGravity gravity);
 
@@ -175,8 +209,6 @@ void xfdesktop_icon_view_set_show_tooltips(XfdesktopIconView *icon_view,
                                            gboolean show_tooltips);
 gint xfdesktop_icon_view_get_tooltip_icon_size(XfdesktopIconView *icon_view);
 
-GtkWidget *xfdesktop_icon_view_get_window_widget(XfdesktopIconView *icon_view);
-
 void xfdesktop_icon_view_sort_icons(XfdesktopIconView *icon_view,
                                     GtkSortType sort_type);
 
@@ -186,6 +218,15 @@ gboolean xfdesktop_icon_view_get_next_free_grid_position(XfdesktopIconView *icon
                                                          gint *next_row,
                                                          gint *next_col);
 
+// This is used only for migration from previous icon position configuration formats
+gboolean xfdesktop_icon_view_grid_geometry_for_metrics(XfdesktopIconView *icon_view,
+                                                       GdkRectangle *total_workarea,
+                                                       GdkRectangle *monitor_workarea,
+                                                       gint *first_row,
+                                                       gint *first_col,
+                                                       gint *last_row,
+                                                       gint *last_col);
+
 G_END_DECLS
 
 #endif  /* __XFDESKTOP_ICON_VIEW_H__ */
